@@ -6,9 +6,13 @@ from publish import command_publish
 from date import command_date
 import listing
 
+__author__ = 'Henry Precheur <henry@precheur.org>'
+__version__ = '1.1'
+__license__ = 'ISCL'
+
 __all__ = ('Post', 'PostError', 'listing', 'jinja_environment',
            'load_configuration', 'load_post_list', 'html_full_url',
-           'command_publish', 'command_date', 'command_check_url')
+           'command_publish', 'command_date')
 
 def main():
     import doctest
