@@ -1,132 +1,114 @@
+import re
 import email
 import codecs
 import logging
 from os import stat
-from datetime import datetime, date
+import datetime
 from urllib import quote
-from cgi import escape
 
-from utils import encode, escape_and_encode
+try:
+    from markdown2 import markdown
+except ImportError:
+    def markdown(text, *args):
+        logging.warning('Markdown syntax not available.'
+                        'Please install markdown2.')
+        # Ugly but better than nothing :)
+        return '<pre>%s</pre>' % text
+    
+
+from html_to_xhtml import html_to_xhtml
 
 class PostError(Exception):
     '''
     Error in post file
     '''
-    def __init__(self, message, filename, line=None):
-        super(PostError, self).__init__(self, message)
-        self.message = message
-        self.filename = filename
-        self.line = line
+    def __init__(self, filename, message):
+        Exception.__init__(self, '%s: %s' % (filename, message))
 
-    def __repr__(self):
-        return '%s(%r, %r, %r)' % (self.__class__.__name__, self.exception,
-                self.filename, self.line)
 
-    def __str__(self):
-        if self.line is None:
-            return 'Error in post file %s: %s' % (self.filename, self.message)
+class Author(unicode):
+
+    _AUTHOR_REGEX = re.compile(u'(?P<name>.+[^\s])\s*<(?P<email>[^@]+@.+)>',
+                               re.UNICODE)
+
+    def name(self):
+        r = self._AUTHOR_REGEX.match(self)
+        if r:
+            return r.group('name')
         else:
-            return 'Error in post file %s line %d: %s' % (self.filename,
-                    self.line, self.message)
+            return self
+
+    def email(self):
+        r = self._AUTHOR_REGEX.match(self)
+        if r:
+            return r.group('email')
+        else:
+            return ''
+
 
 class Post(object):
 
-    DEFAULT_ENCODING = 'ascii'
-    DEFAULT_AUTHOR = 'unknown author'
+    DEFAULT_ENCODING = u'ascii'
+    DEFAULT_AUTHOR = u'unknown author'
 
-    def __init__(self, f):
-        '''
-        >>> file_content = """title: test
-        ... date: 2008-1-1
-        ... author: test author
-        ... encoding: utf-8
-        ... 
-        ... test."""
-        >>> from StringIO import StringIO
-        >>> p = Post(StringIO(file_content))
-        >>> p
-        <Post('test', datetime.date(2008, 1, 1))>
-        >>> p.title == 'test'
-        True
-        >>> import datetime
-        >>> p.date == datetime.date(2008, 1, 1)
-        True
-        >>> p.content == 'test.'
-        True
-        >>> p.encoding == 'utf-8'
-        True
-        >>> p.author == 'test author'
-        True
-        >>> Post(StringIO('title: no payload\\ndate: 2008-1-1'))
-        <Post('no payload', datetime.date(2008, 1, 1))>
-        >>> Post(StringIO('title: no date'))
-        Traceback (most recent call last):
-        ...
-        PostError: Error in post file <unknown filename>: No date defined
-        >>> Post(StringIO("""title: bad encoding
-        ... date: 2008-1-1
-        ... encoding: bad-encoding"""))
-        Traceback (most recent call last):
-        ...
-        PostError: Error in post file <unknown filename>: unknown encoding: \
-bad-encoding
-        >>> Post(StringIO("""title: bad date
-        ... date: 200008-101-10"""))
-        Traceback (most recent call last):
-        ...
-        PostError: Error in post file <unknown filename>: Unable to parse \
-date '200008-101-10'
-        (Use YYYY-MM-DD [[HH:MM]:SS] format)
-        '''
-        if isinstance(f, (str, basestring)):
+    def __init__(self, f, markup=None):
+        if isinstance(f, basestring):
             self._filename = f
             input_file = open(f)
         else:
             self._filename = None
             input_file = f
         post_file = email.message_from_file(input_file)
-        self.__dict__.update((key.lower(), value) for (key, value) in
-                             post_file.items())
-        if not hasattr(self, 'encoding'):
-            self.encoding = self.DEFAULT_ENCODING
-        if self.encoding.lower() != 'raw':
-            try:
-                codecs.lookup(self.encoding)
-            except LookupError, e:
-                raise PostError(e.message, self.get_filename())
+
+        # First get the file's encoding
+        self.encoding = unicode(post_file.get('encoding') or
+                                self.DEFAULT_ENCODING)
+        try:
+            codecs.lookup(self.encoding)
+        except LookupError, e:
+            raise PostError(self.get_filename(), str(e))
+
+        # Copy all field "into the object" and convert string to unicode.
+        try:
+            for key, value in post_file.items():
+                self.__dict__[key.encode('ascii').lower()] = \
+                        unicode(value, self.encoding)
+        except UnicodeDecodeError, e:
+            raise PostError(self.get_filename(), "for key '%s': %s" % (key, e))
+
         if not hasattr(self, 'author'):
-            self.author = self.DEFAULT_AUTHOR
+            self.author = Author(self.DEFAULT_AUTHOR)
+        else:
+            self.author = Author(self.author)
+
         # Handle the date. If no date was specified use the file's modification
         # time.
         if not hasattr(self, 'date'):
-            # Get the date from file's mtime and issue a warning
             if not self._filename:
-                raise PostError('No date defined', self.get_filename())
+                self.date = None
             else:
-                self.date = datetime.\
-                            fromtimestamp(stat(self._filename).st_mtime)
-                logging.warning('No date defined in \'%s\', using the ' \
-                                'file\'s last modification time instead.' % \
+                # Get the date from file's mtime and issue a warning
+                mtime = stat(self._filename).st_mtime
+                self.date = datetime.datetime.fromtimestamp(mtime)
+                logging.warning("No date defined in '%s', using the file's "
+                                "last modification time instead." % \
                                 self._filename)
         else:
             try:
                 self.date = self.parse_date(self.date)
             except ValueError, e:
-                raise PostError(e.message, self.get_filename())
+                raise PostError(self.get_filename(), str(e))
+
         try:
-            self.ascii_title = self.title.decode('ascii'
-                                                 if self.encoding == 'raw'
-                                                 else self.encoding).\
-                               encode('ascii', 'replace')
-            self.title = escape_and_encode(self.title, self.encoding)
+            self.ascii_title = self.title.encode('ascii', 'replace')
         except UnicodeDecodeError, e:
-            raise PostError('Bad encoding in title', self.get_filename())
+            raise PostError(self.get_filename(), 'Bad encoding in title')
+
+        if not post_file.get_payload():
+            raise PostError(self.get_filename(), 'does not have content')
+
         try:
-            self.author = escape_and_encode(self.author, self.encoding)
-        except UnicodeDecodeError, e:
-            raise PostError('Bad encoding in author', self.get_filename())
-        try:
-            self.content = encode(post_file.get_payload(), self.encoding)
+            self.content = unicode(post_file.get_payload(), self.encoding)
         except UnicodeDecodeError, e:
             # find error line number
             for line_number, line in enumerate(post_file.as_string().\
@@ -137,9 +119,25 @@ date '200008-101-10'
                 except UnicodeDecodeError, e:
                     break
             # line_number starts at 0, real line number == line_number + 1
-            raise PostError('Bad encoding in content line %d, %s' % \
-                            (line_number + 1, e),
-                            self.get_filename())
+            raise PostError(self.get_filename(),
+                            'Bad encoding in content line %d, %s' % \
+                            (line_number + 1, e))
+
+        if not markup: # Determine type via file extension
+            if self._filename and self._filename.endswith('.txt'):
+                self._markup = 'markdown'
+            elif self._filename and self._filename.endswith('.html'):
+                self._markup = 'html'
+            elif not self._filename:
+                self._markup = 'html'
+            else:
+                logging.warning("Unable to determine '%s' type, falling "
+                                "back to HTML" % self._filename)
+                self._markup = 'html'
+        else:
+            assert markup in ('markdown', 'html')
+            self._markup = markup
+
         # Transform the 'files' field into a list of string
         if hasattr(self, 'files'):
             self.files = self.files.split()
@@ -196,15 +194,15 @@ date '200008-101-10'
         """
         for date_format in Post._DATE_FORMAT_LIST:
             try:
-                return datetime.strptime(date_, date_format).date()
+                return datetime.datetime.strptime(date_, date_format).date()
             except ValueError:
                 continue
         for date_format in Post._DATETIME_FORMAT_LIST:
             try:
-                return datetime.strptime(date_, date_format)
+                return datetime.datetime.strptime(date_, date_format)
             except ValueError:
                 continue
-        raise ValueError('Unable to parse date %r\n'
+        raise ValueError('Unable to parse date \'%s\'\n'
                          '(Use YYYY-MM-DD [[HH:MM]:SS] format)' %
                          (date_))
 
@@ -214,14 +212,26 @@ date '200008-101-10'
         else:
             return self._filename
 
+    def get_html(self):
+        if self._markup == 'markdown':
+            return markdown(self.content, html4tags=True)
+        elif self._markup == 'html':
+            return self.content
+        else:
+            assert False, "Unknown type for %r" % self
+
+    def get_xhtml(self):
+        if self._markup == 'markdown':
+            return markdown(self.content, html4tags=False)
+        elif self._markup == 'html':
+            return html_to_xhtml(self.content)
+        else:
+            assert False, "Unknown type for %r" % self
+
     def __cmp__(self, other):
         '''
-        >>> file1 = """title: 1
-        ... date: 2008-1-1
-        ...
-        ... test"""
-        >>> file2 = """title: 2
-        ... date: 2007-12-31"""
+        >>> file1 = "title: 1\\ndate: 2008-1-1\\n\\ntest"
+        >>> file2 = "title: 2\\ndate: 2007-12-31\\n\\ntest"
         >>> from StringIO import StringIO
         >>> Post(StringIO(file1)) > Post(StringIO(file2))
         True
@@ -233,8 +243,8 @@ date '200008-101-10'
         >>> l.index(Post(StringIO(file1)))
         1
         '''
-        return cmp(str(self.date) + str(self.title),
-                   str(other.date) + str(self.title))
+        return cmp(unicode(self.date) + self.title,
+                   unicode(other.date) + other.title)
 
     def __hash__(self):
         return hash(str(self.date) + self.title)
