@@ -1,10 +1,11 @@
 import os
 import datetime
 import logging
+import codecs
 from shutil import copy
 
+import weblog
 from _jinja_environment import jinja_environment
-from PyRSS2Gen import RSS2, RSSItem
 from html_full_url import html_full_url
 from post import Post, PostError
 from load import load_post_list, load_configuration
@@ -22,32 +23,17 @@ def generate_post_html(post_list, output_dir, post_tmpl, params):
             os.makedirs(dir)
         elif not os.path.isdir(dir):
             raise IOError('\'%s\' already exists and is not a directory' % dir)
-        filename = os.path.join(dir, post.ascii_title + '.html')
-        output = file(filename, 'w')
         top_dir = '../../../'
-        output.write(post_tmpl.render(title=post.title,
-                                      date=post.date,
-                                      author=post.author,
-                                      content=html_full_url(top_dir,
-                                                            post.content),
-                                      top_dir=top_dir,
-                                      **dict(((k, v) for k, v in
-                                              params.iteritems()
-                                              if k != 'title'))))
-
-def generate_rss(post_list, filename, params):
-    def make_rss_item(post):
-        return RSSItem(title=post.title,
-                       link=post.url(prefix=params['url']),
-                       description=html_full_url(params['url'], post.content),
-                       guid=post.url(prefix=params['url']),
-                       pubDate=post.date)
-    rss = RSS2(title = params['title'],
-               link = params['url'],
-               description = params['description'],
-               lastBuildDate = datetime.datetime.now(),
-               items=(make_rss_item(post) for post in post_list))
-    rss.write_xml(open(filename, "w"))
+        r = post_tmpl.render(title=post.title,
+                             date=post.date,
+                             author=post.author,
+                             content=html_full_url(top_dir,
+                                                   post.get_html()),
+                             top_dir=top_dir,
+                             **dict(((k, v) for k, v in
+                                     params.iteritems()
+                                     if k != 'title' and k != 'content')))
+        open(os.path.join(dir, post.ascii_title + '.html'), 'w').write(r)
 
 def command_publish(args, options):
     source_dir = options.source_dir
@@ -78,11 +64,11 @@ def command_publish(args, options):
         post_list = list(reversed(sorted(load_post_list(source_dir))))
     except (IOError, PostError), e:
         logging.error('Error while loading post files.')
-        exit(e)
+        raise SystemExit(e)
 
     def generate_all():
         params = dict(title=config['title'],
-                      description=config['description'],
+                      description=config.get('description'),
                       url=config['url'],
                       html_head=config.get('html_head'),
                       html_header=config.get('html_header'),
@@ -99,8 +85,7 @@ def command_publish(args, options):
 
         logging.debug('Generating HTML posts files')
         post_tmpl = env.get_template('post.html.tmpl')
-        generate_post_html(post_list, output_dir, post_tmpl,
-            params)
+        generate_post_html(post_list, output_dir, post_tmpl, params)
         # Copy all 'attached' files
         for post in post_list:
             for filename in post.files:
@@ -112,8 +97,23 @@ def command_publish(args, options):
                     os.makedirs(destination_dir)
                 copy(os.path.join(source_dir, filename), destination)
 
-        generate_rss(post_list[:config['rss_limit']],
-                os.path.join(output_dir, 'rss.xml'), params)
+        # Generate Atom feed
+        template = env.get_template('feed.atom.tmpl')
+
+        # Last time the feed was updated
+        posts = post_list[:config['feed_limit']]
+        if posts:
+            feed_updated = max(p.date for p in posts)
+        else:
+            feed_updated = datetime.datetime.utcnow()
+
+        atom_file = codecs.open(os.path.join(output_dir, 'feed.atom'), 'w',
+                                encoding='utf8')
+        atom_file.write(template.render(posts=posts,
+                                        feed_updated=feed_updated,
+                                        weblog_version=weblog.__version__,
+                                        **params))
+        atom_file.close()
 
         for f in config.get('extra_files', '').split():
             copy(os.path.join(source_dir, f), output_dir)
@@ -125,6 +125,6 @@ def command_publish(args, options):
             generate_all()
         except IOError, e:
             logging.error('Error while generating files ...')
-            exit(e)
+            raise SystemExit(e)
         else:
             logging.info('Successfully generated weblog.')
